# -*- coding: utf-8 -*-
# @Time    : 2019/11/21 14:38
# @Author  : Dong Qirui
# @Software: PyCharm

# # 方法限时功能
# from .timeout_decorator import *
#
# __all__ = list( filter( lambda x : not str( x ).__contains__( '__' ), locals().keys() ) )



# -*- coding: utf-8 -*-
# @Time    : 2019/11/21 14:43
# @Author  : Dong Qirui
# @Software: PyCharm
import json
import logging
import sys

from QrCerUtils.decorators.timeout.decorator_timeout import timeout
from QrCerUtils.utils.wechat_work.util_robot_message_builder import message_builder


class robot_message_sender :
    
    def __init__ ( self, url: str ) -> None :
        self.url = url
        super().__init__()
    
    @timeout( 30 )
    def notification_wechat_work ( payload: json, key: str ) -> json :
        '''
    
        :param payload:
        :param key:
        :return:
        '''
        try :
            import requests
            
            url = "https://qyapi.weixin.qq.com/cgi-bin/webhook/send"
            querystring = {"key" : key}
            
            resp = requests.post( url, data=payload, headers=GLOBAL_HEADERS, params=querystring )
            logging.info( resp.text )
        except :
            logging.error( sys._getframe().f_code.co_name, exc_info=True )
    
    @timeout( 30 )
    def notification_wechat_work_markdown ( message: str, key: str ) -> json :
        '''
        
        :param message:
        :param key:
        :return:
        '''
        try :
            return notification_wechat_work( message_builder.markdown( message ), key )
        except :
            logging.error( sys._getframe().f_code.co_name, exc_info=True )



# -*- coding: utf-8 -*-
# @Time    : 2019/08/24 01:24
# @Author  : Dong Qirui
# @FileName: logger_utils.py
# @Software: PyCharm

import logging
import sys
from logging import handlers

sys


def setLogger () :
    '''
    
    :return:
    '''
    LOGGER = logging.getLogger()
    LOGGER.setLevel( logging.INFO )
    TIME_HANDLER = logging.handlers.TimedRotatingFileHandler( filename='project.log', when='D', interval=1, backupCount=2, encoding='utf-8' )
    # 必须是这种格式才能自动删除
    TIME_HANDLER.suffix = '%Y-%m-%d.log'
    LOGGER.addHandler( TIME_HANDLER )
    LOGGER.addHandler( logging.StreamHandler() )


setLogger()



# -*- coding: utf-8 -*-
# @Time    : 2019/08/24 01:41
# @Author  : Dong Qirui
# @FileName: wechat_work_builder.py
# @Software: PyCharm
import json
import logging
from typing import List


class message_builder() :
    
    def __init__ ( self ) :
        pass
    
    class Constants() :
        MESSAGE_TYPE = 'msgtype'
        MESSAGE_TYPE_TEXT = 'text'
        MESSAGE_TYPE_MARKDOWN = 'markdown'
        MESSAGE_TYPE_IMAGE = 'image'
        MESSAGE_TYPE_NEWS = 'news'
        MESSAGE_CONTENT = 'content'
    
    @staticmethod
    def text ( message_text_content: str, message_text_mentioned_list: list = None, message_text_mentioned_mobile_list: list = None ) -> json :
        '''
        构建文本内容
        :param message_text_content:
        :param message_text_mentioned_list:
        :param message_text_mentioned_mobile_list:
        :return:
        '''
        if len( message_text_content ) > 2048 :
            logging.error( '文本内容，最长不超过2048个字节，必须是utf8编码' )
        message = json.dumps(
                {
                        message_builder.Constants.MESSAGE_TYPE      : message_builder.Constants.MESSAGE_TYPE_TEXT,
                        message_builder.Constants.MESSAGE_TYPE_TEXT : {
                                message_builder.Constants.MESSAGE_CONTENT : message_text_content, 'mentioned_list' : message_text_mentioned_list,
                                'mentioned_mobile_list'                   : message_text_mentioned_mobile_list
                                }
                        } )
        return message
    
    @staticmethod
    def markdown ( message_markdown_content: str ) -> json :
        '''
        构建MarkDown内容
        :param message_markdown_content: markdown内容，最长不超过4096个字节，必须是utf8编码
        :return:
        '''
        message = json.dumps( {
                message_builder.Constants.MESSAGE_TYPE          : message_builder.Constants.MESSAGE_TYPE_MARKDOWN,
                message_builder.Constants.MESSAGE_TYPE_MARKDOWN : {message_builder.Constants.MESSAGE_CONTENT : message_markdown_content}
                } )
        return message
    
    @staticmethod
    def image ( message_image_base64: str, message_image_md5: str ) -> json :
        '''
        构建图片内容
        :param message_image_base64: 图片内容的base64编码, 图片（base64编码前）最大不能超过2M，支持JPG,PNG格式
        :param message_image_md5: 图片内容（base64编码前）的md5值
        :return:
        '''
        message = json.dumps( {
                message_builder.Constants.MESSAGE_TYPE       : message_builder.Constants.MESSAGE_TYPE_IMAGE,
                message_builder.Constants.MESSAGE_TYPE_IMAGE : {'base64' : message_image_base64, 'md5' : message_image_md5}
                } )
        return message
    
    @staticmethod
    def news ( message_news_articles: List[dict] ) -> json :
        '''
        构建图文内容
        :param message_news_articles: 图文消息，一个图文消息支持1到8条图文
        :return:
        '''
        message = json.dumps(
                {message_builder.Constants.MESSAGE_TYPE : message_builder.Constants.MESSAGE_TYPE_NEWS, message_builder.Constants.MESSAGE_TYPE_NEWS : {'articles' : message_news_articles}} )
        return message
    
    @staticmethod
    def news_article ( title: str, url: str = '', description: str = None, picurl: str = None ) -> dict :
        '''
        构建图文新闻
        :param title: 标题，不超过128个字节，超过会自动截断
        :param url: 点击后跳转的链接。
        :param description: 描述，不超过512个字节，超过会自动截断
        :param picurl: 图文消息的图片链接，支持JPG、PNG格式，较好的效果为大图 1068*455，小图150*150。
        :return:
        '''
        url = 'family.ke.com' if None is url or '' == url else url
        article = {'title' : title, 'url' : url, 'description' : description, 'picurl' : picurl}
        return article
    
    @staticmethod
    def markdown_merge ( request_json: str, applicationName: str ) -> json :
        '''
    
        :param request_json:
        :return:
        '''
        if 'merge_request' == request_json['object_kind'] :
            name_operator = request_json['user']['name']
            username_operator = request_json['user']['username']
            merge_status = request_json['object_attributes']['merge_status']
            state = request_json['object_attributes']['state']
            title = request_json['object_attributes']['title']
            description = request_json['object_attributes']['description']
            source_branch = request_json['object_attributes']['source_branch']
            target_branch = request_json['object_attributes']['target_branch']
            name_assignee = request_json['assignee']['name'] if 'assignee' in request_json and 'name' in request_json['assignee'] else '无'
            username_assignee = request_json['assignee']['username'] if 'assignee' in request_json and 'username' in request_json['assignee'] else None
            target_branch_translate = {
                    'dev'    : '<font color=\"info\">dev</font>',
                    'master' : '<font color=\"red\">master (请谨慎合入)</font>',
                    }.get( target_branch, target_branch )
            url = request_json['object_attributes']['url']
            action = request_json['object_attributes'].get( 'action', 'modify' )
            action_translate = {
                    'open'   : '<font color=\"info\">发起</font>',
                    'close'  : '<font color=\"warning\">关闭</font>',
                    'update' : '<font color=\"warning\">更新</font>',
                    'modify' : '<font color=\"warning\">操作</font>',
                    'merge'  : '<font color=\"red\">合入</font>'
                    }.get( action )
            str_markdown = '''
\r\n### **{name_operator}**{action_translate}了一个Code Review
\r\n##### <font color=\"comment\">主要描述: </font>{title}
\r\n###### <font color=\"comment\">详细内容: </font>{description}
\r\n###### <font color=\"comment\">来源分支: </font>{source_branch}
\r\n###### <font color=\"comment\">目标分支: </font>{target_branch_translate}
\r\n###### <font color=\"comment\">评审指派: </font>{name_assignee}
\r\n###### <font color=\"comment\">访问地址: </font>[点我访问]({url})
            '''.format( name_operator=name_operator, title=title, merge_status=merge_status, url=url, target_branch_translate=target_branch_translate, action=action,
                        action_translate=action_translate, description='\r\n' + description if description.__contains__( '-' ) or description.__contains__( '#' ) else description,
                        source_branch=source_branch, name_assignee=name_assignee )
            if (applicationName == 'Application_Layer') and (title.__contains__( '000' ) or description.__contains__( '000' )) :
                str_markdown += '\r\n###### 如果包含接口配置更新，请及时通知QA同学添加TestCase进行测试，并于封板前同步线上策略规则！'
            
            return message_builder.markdown( str_markdown )



# -*- coding: utf-8 -*-
# @Time    : 2019/08/24 01:10
# @Author  : Dong Qirui
# @FileName: timeout_decorator.py
# @Software: PyCharm

# from kthread import *
import threading


class KThread( threading.Thread ) :
    """
    A subclass of threading.Thread, with a kill()
    method.
    Come from:
    Kill a thread in Python:
    http://mail.python.org/pipermail/python-list/2004-May/260937.html
    """
    
    def __init__ ( self, *args, **kwargs ) :
        threading.Thread.__init__( self, *args, **kwargs )
        self.killed = False
    
    def start ( self ) :
        
        """Start the thread."""
        self.__run_backup = self.run
        self.run = self.__run  # Force the Thread to install our trace.
        threading.Thread.start( self )
    
    def __run ( self ) :
        
        """Hacked run function, which installs the
        trace."""
        # sys.settrace(self.globaltrace)
        self.__run_backup()
        self.run = self.__run_backup
    
    def globaltrace ( self, frame, why, arg ) :
        if why == 'call' :
            return self.localtrace
        else :
            return None
    
    def localtrace ( self, frame, why, arg ) :
        if self.killed :
            if why == 'line' :
                raise SystemExit()
        return self.localtrace
    
    def kill ( self ) :
        self.killed = True


class Timeout( Exception ) :
    """function run timeout"""


def timeout ( seconds ) :
    '''
    超时装饰器，指定超时时间
    若被装饰的方法在指定的时间内未返回，则抛出Timeout异常
    :param seconds:
    :return:
    '''
    
    def timeout_decorator ( func ) :
        """真正的装饰器"""
        
        def _new_func ( oldfunc, result, oldfunc_args, oldfunc_kwargs ) :
            result.append( oldfunc( *oldfunc_args, **oldfunc_kwargs ) )
        
        def _ ( *args, **kwargs ) :
            result = []
            new_kwargs = {  # create new args for _new_func, because we want to get the func return val to result list
                    'oldfunc'        : func,
                    'result'         : result,
                    'oldfunc_args'   : args,
                    'oldfunc_kwargs' : kwargs
                    }
            thd = KThread( target=_new_func, args=(), kwargs=new_kwargs )
            thd.start()
            thd.join( seconds )
            alive = thd.is_alive()
            thd.kill()  # kill the child thread
            if alive :
                # raise Timeout(u'function run too long, timeout %d seconds.' % seconds)
                try :
                    raise Timeout( u'function run too long, timeout %d seconds.' % seconds )
                finally :
                    return u'function run too long, timeout %d seconds.' % seconds
            elif len( result ) > 0 :
                return result[0]
            else :
                return None
        
        _.__name__ = func.__name__
        _.__doc__ = func.__doc__
        return _
    
    return timeout_decorator



# -*- coding: utf-8 -*-
# @Time    : 2019/11/21 16:04
# @Author  : Dong Qirui
# @Software: PyCharm


import logging
import sys

from QrCerUtils.decorators.timeout.decorator_timeout import timeout


@timeout( 30 )
def image_limit_size ( content: bytes, limit_size: int = 1600000 ) -> bytes :
    '''
    限制图像尺寸
    :param content:
    :return:
    '''
    try :
        from io import BytesIO
        from PIL import Image
        
        if not len( content ) > limit_size :
            pass
        im = Image.open( BytesIO( content ) )
        limit = len( content ) / limit_size
        image = im.resize( (int( im.width / limit ), int( im.height / limit )), Image.NEAREST )
        image_bytes = BytesIO()
        image.save( image_bytes, format='JPEG' )
        content_resized = image_bytes.getvalue()
        print( len( content_resized ) )
        return content_resized
    except :
        logging.error( sys._getframe().f_code.co_name, exc_info=True )
        return None



# -*- coding: utf-8 -*-
# @Time    : 2019/11/21 14:56
# @Author  : Dong Qirui
# @Software: PyCharm


from QrCerUtils.utils import getTodayDateStrftimeShort


class redis_instance() :
    
    def __init__ ( self ) :
        import redis
        
        # self.redis_instance = redis.StrictRedis( host='127.0.0.1', port=6379, password='', db=0 )
        self.redis_instance = redis.StrictRedis( host='127.0.0.1', port=6379, password='password', db=0 )
    
    def instance ( self ) :
        '''
        
        :return:
        '''
        return self.redis_instance
    
    def hash_get ( self, key: str, field: str ) -> str :
        '''
        
        :param key:
        :param field:
        :return:
        '''
        return self.redis_instance.hget( key, field )
    
    def hash_exists_robot ( self, field: str ) -> str :
        '''
        
        :param field:
        :return:
        '''
        return self.redis_instance.hexists( self.KEY_BASE, field )
    
    def hash_get_robot ( self, field: str ) -> str :
        '''
        
        :param key:
        :param field:
        :return:
        '''
        return self.redis_instance.hget( self.KEY_BASE, field )
    
    def hash_set_robot ( self, field: str, value: str ) -> str :
        '''
        
        :param key:
        :param field:
        :return:
        '''
        return self.redis_instance.hset( self.KEY_BASE, field, value )
    
    def redis_hash_get_robot_daily ( self, field: str ) -> str :
        '''
        
        :param key:
        :param field:
        :return:
        '''
        date_str = getTodayDateStrftimeShort()
        return self.redis_instance.hget( self.KEY_BASE, '{date}_{field}'.format( date=date_str, field=field ) )
    
    def hash_exists_robot_daily ( self, field: str ) -> str :
        '''
        
        :param key:
        :param field:
        :return:
        '''
        date_str = getTodayDateStrftimeShort()
        return self.redis_instance.hexists( self.KEY_BASE, '{date}_{field}'.format( date=date_str, field=field ) )



# -*- coding: utf-8 -*-
# @Time    : 2019/11/21 14:52
# @Author  : Dong Qirui
# @Software: PyCharm

import logging

from QrCerUtils.decorators.timeout.decorator_timeout import timeout


@timeout( 30 )
def send_email ( subject: str, body: str, to_recipients: list = ['DongQiRui001@Ke.com'], cc_recipients: list = [] ) :
    '''
    发送邮件，有明文密码，代码不可泄漏
    :return:
    '''
    try :
        from exchangelib import DELEGATE, Account, Credentials, Configuration, NTLM, Message, Mailbox, HTMLBody
        from exchangelib.protocol import BaseProtocol, NoVerifyHTTPAdapter
        
        BaseProtocol.HTTP_ADAPTER_CLS = NoVerifyHTTPAdapter
        cred = Credentials( r'LianJia\DongQiRui001', 'password' )
        config = Configuration( server='Mail.ke.com', credentials=cred, auth_type=NTLM )
        account = Account( primary_smtp_address='DongQiRui001@Ke.com', config=config, autodiscover=False, access_type=DELEGATE )
        message = Message(
                account=account,
                folder=account.sent,
                subject=subject,
                body=HTMLBody( body ),
                to_recipients=map( lambda x : Mailbox( email_address=x ), to_recipients ) if None is not to_recipients and len( to_recipients ) > 0 else [],
                cc_recipients=map( lambda x : Mailbox( email_address=x ), cc_recipients ) if None is not cc_recipients and len( cc_recipients ) > 0 else []
                )
        message.send_and_save()
    except :
        logging.error( sys._getframe().f_code.co_name, exc_info=True )
        return None



# -*- coding: utf-8 -*-
# @Time    : 2019-01-03 16:29
# @Author  : Dong Qirui
# @FileName: keyscan.py.py
# @Software: PyCharm


import argparse
import optparse
import re
import sys

import redis


class Counter( object ) :
    
    def __init__ ( self, host, port, password=None, db=0, sample_rate=1.0 ) :
        self.host = host
        self.port = port
        self.password = password
        self.db = db
        self.limit = 0
        self.sample_rate = min( max( sample_rate, 0.1 ), 1.0 )
        self._setup_()
        self._info_()
        self._run_()
        self.show()
    
    def _setup_ ( self ) :
        try :
            self.conn = redis.StrictRedis( host=self.host, port=self.port, password=self.password, db=self.db )
            self.db_size = self.conn.dbsize()
            self.limit = int( self.sample_rate * self.db_size )
            self.pipeline = self.conn.pipeline()
            self.iter_counts = 0
            self.result = dict()
            self.pattern = re.compile( '((\D+)((_\d*)+))' )
            if self.limit <= 0 :
                print( 'Count is zero, exit now.' )
        except Exception as e :
            print( repr( e ) )
            sys.exit( 0 )
    
    def _info_ ( self ) :
        print( 'INFO {0}:{1}'.format( 'host', self.host ) )
        print( 'INFO {0}:{1}'.format( 'port', self.port ) )
        print( 'INFO {0}:{1}'.format( 'password', self.password[:2] + '*' ) )
        print( 'INFO {0}:{1}'.format( 'db', self.db ) )
        print( 'INFO {0}:{1}'.format( 'sample_rate', self.sample_rate ) )
    
    def _run_ ( self ) :
        print( 'PROCESS start.' )
        for key in self.conn.scan_iter( match='*', count=100000 ) :
            self.iter_counts += 1
            if self.iter_counts % 10000 == 0 :
                print( '{0}%'.format( round( 100.0 * self.iter_counts / self.limit ) ) )
            if self.iter_counts >= self.limit :
                return
            try :
                grp = re.search( self.pattern, key )
                if None != grp :
                    prefix = re.search( self.pattern, key ).groups()[1]
                    if prefix in self.result :
                        self.result[prefix] += 1
                    else :
                        self.result[prefix] = 1
            except Exception as e :
                print( repr( e ) )
                sys.exit( 0 )
        print( 'PROCESS end.' )
    
    def show ( self ) :
        print( ''.join( map( lambda e : '{0}\t{1}\n'.format( e[0], e[1] ), self.result.items() ) ) )


def main () :
    usage = '''Usage: %prog [options] arg1 arg2 ...
    
    用于统计Redis中Key的计数
    count key in redis...
    
    EXAMPLE: python redis_key_util.py --host 10.33.108.52 --port 6379 -d 0 -a $password
    使用示例：python redis_key_util.py --host 10.33.108.52 --port 6379 -d 0 -a $password
    '''
    epilog = """

    """
    if sys.version_info > (2, 7) :
        parser = optparse.OptionParser( usage=usage, epilog=epilog, version='%prog 1.0' )
        parser.add_option( '--host', dest='host', help='host of the redis server' )
        parser.add_option( '--port', dest='port', type=int, help='port of the redis server' )
        parser.add_option( '-d', type=int, dest='db', help='the database, which default is 0' )
        parser.add_option( '-a', dest='password', help='the password, which default is None, if the redis server ask for authentication' )
        parser.add_option( '-s', dest='sample_rate', help='the sample size, which default is 1.0, means all of the db_size' )
        (options, args) = parser.parse_args()
        if None == options.host :
            parser.error( 'Parameter %s required' % 'host' )
        if None == options.port :
            parser.error( 'Parameter %s required' % 'port' )
    else :
        parser = argparse.ArgumentParser( usage='用于统计Redis中Key的计数', description='this script sample and analyze the key counts of the redis server, group by common key prefix', epilog=epilog,
                                          formatter_class=argparse.RawTextHelpFormatter, add_help=True )
        parser.add_argument( '--host', help='host of the redis server' )
        parser.add_argument( '--port', type=int, help='port of the redis server' )
        parser.add_argument( '-d', type=int, dest='db', help='the database, which default is 0' )
        parser.add_argument( '-a', dest='password', help='the password, which default is None, if the redis server ask for authentication' )
        parser.add_argument( '-s', type=float, dest='sample_rate', help='the sample size, which default is 1.0, means all of the db_size' )
        options = parser.parse_args()
        args = parser.parse_args()
    print( options.host, options.port, options.db, options.password, options.sample_rate )
    Counter( options.host, options.port, password=options.password, sample_rate=options.sample_rate, db=options.db )


if __name__ == '__main__' :
    main()



# -*- coding: utf-8 -*-
# @Time    : 2020/11/05 2:10 PM
# @Author  : Dong Qirui
# @FileName: encode_utils.py
# @Software: PyCharm
# @Version : 0.10

# How-To-Use：
# In [3]: from QrCerUtils.utils.encode import util_encode_detect
#
# In [4]: str_a =  b'\xc8\xab\xb2\xbf\xb3\xc9\xbd\xbb'
#
# In [5]: util_encode_detect.fix_coding(str_a)
# Out[5]: '全部成交'
#
# In [6]: util_encode_detect.get_coding(str_a)
# Out[6]: 'gbk'

import chardet


def get_coding ( str_input ) :
    '''
    获取编码格式
    :param str_input:
    :return:
    '''
    encoding: str = None
    try :
        if isinstance( str_input, bytes ) :
            encoding = chardet.detect( str_input )['encoding']
        elif isinstance( str_input, str ) :
            str_encode = str_input.encode( 'latin1' )
            encoding = chardet.detect( str_encode )['encoding']
        encoding = 'gbk' if encoding.__contains__( 'GB' ) else encoding
    except :
        pass
    return encoding


def fix_coding ( str_input ) :
    '''
    转换为真实编码
    :param str_input:
    :return:
    '''
    encoding: str = get_coding( str_input )
    str_decode = str_input
    try :
        if None is encoding :
            str_decode = str( str_input )
        if isinstance( str_input, bytes ) :
            str_decode = str( str_input, encoding )
        elif isinstance( str_input, str ) :
            str_encode = str_input.encode( 'latin1' )
            str_decode = str_encode.decode( encoding )
    except :
        pass
    return str_decode



# -*- coding: utf-8 -*-
# @Time    : 2020/10/21 05:03
# @Author  : Dong Qirui
# @Software: PyCharm


def tail_file ( thefile, interval=0.002 ) :
    '''
    监控文件改动
    :param thefile:
    :param interval: 监控间隔，默认值0.002 = 2ms
    :return:
    '''
    import time
    
    thefile.seek( 0, 2 )
    while True :
        line_str = thefile.readline()
        if not line_str :
            time.sleep( interval )
            continue
        yield line_str



# -*- coding: utf-8 -*-
# @Time    : 2018/11/12 4:12 PM
# @Author  : Dong Qirui
# @FileName: encode_utils.py
# @Software: PyCharm
# @Version  : 0.10
# 已经过时，建议使用util_encode_detect

def get_coding ( strInput ) :
    '''
    获取编码格式
    :param strInput:
    :return:
    '''
    import sys
    
    if sys.version_info.major == 3 :
        unicode = str
    if isinstance( strInput, unicode ) :
        return "unicode"
    try :
        strInput.decode( "utf8" )
        return 'utf8'
    except :
        pass
    try :
        strInput.decode( "gbk" )
        return 'gbk'
    except :
        pass


def trans_utf8 ( strInput ) :
    '''
    转化为utf8格式
    :param strInput:
    :return:
    '''
    strCodingFmt = get_coding( strInput )
    if strCodingFmt == "utf8" :
        return strInput
    elif strCodingFmt == "unicode" :
        return strInput.encode( "utf8" )
    elif strCodingFmt == "gbk" :
        return strInput.decode( "gbk" ).encode( "utf8" )


def trans_gbk ( strInput ) :
    '''
    转化为gbk格式
    :param strInput:
    :return:
    '''
    strCodingFmt = get_coding( strInput )
    if strCodingFmt == "gbk" :
        return strInput
    elif strCodingFmt == "unicode" :
        return strInput.encode( "gbk" )
    elif strCodingFmt == "utf8" :
        return strInput.decode( "utf8" ).encode( "gbk" )


def assert_utf8 ( strInput ) :
    '''
    保证不会是unicode, 否则容易引发错误, python3已经修复默认编码
    :param strInput:
    :return:
    '''
    return trans_utf8( strInput ) if None != get_coding( strInput ) else str( strInput )



# -*- coding: utf-8 -*-
# @Time    : 2020/11/02 14:40
# @Author  : Dong Qirui
# @Software: PyCharm

# from decorators.timeout.timeout_decorator import KThread
# from utils.datetime import datetime_utils
# from utils.email import email_utils
# from utils.encode import automatic_switch_python_version,encode_utils
# from utils.image import image_utils
# from utils.redis import redis_utils,redis_key_util
# from utils.wechat_work import robot_message_builder_util,robot_message_sender_util


# -*- coding: utf-8 -*-
# @Time    : 2020/10/30 18:50
# @Author  : Dong Qirui
# @Software: PyCharm

from setuptools import setup, find_packages, Extension
from distutils.core import setup, Extension

import setuptools

with open( 'ReadMe.md', 'r' ) as fh :
    long_description = fh.read()

setuptools.setup(
        # 名称
        name='QrCerUtils',
        # 版本
        version='0.1.6',
        # 描述
        description="Common utils for python.",
        # 详细描述
        long_description=long_description,
        # 详细描述格式
        long_description_content_type='text/markdown',
        # 关键字
        keywords='common utils python base',
        # 作者
        author='Dong Qirui',
        # 作者邮箱
        author_email='QrCeric@Gmail.com',
        # 作者链接
        url='https://github.com/QrCer/qrcer_commons',
        packages=setuptools.find_packages(),
        # packages=setuptools.find_packages( exclude=['ez_setup', 'examples', 'tests'] ),
        # package name : [file name],
        include_package_data=True,
        zip_safe=False,
        # install_requires = ['docutils>=0.3'],
        install_requires=[  # 需求的第三方模块
                'redis',
                ],
        # entry_points={
        #         'console_scripts' : [  # 如果你想要以Linux命令的形式使用
        #                 'qc = utils.encode.automatic_switch_python_version:main'
        #                 ]
        #         },
        classifiers=[
                'Programming Language :: Python :: 2',
                'Programming Language :: Python :: 3',
                'License :: OSI Approved :: MIT License',
                'Operating System :: OS Independent',
                ],
        )



# -*- coding: utf-8 -*-
# @Time    : 2019/09/16 11:49
# @Author  : Dong Qirui
# @Software: PyCharm
# import logging


import sys

if sys.version_info > (3,) :
    pass
else :
    if sys.version_info >= (2, 7) :
        pass
    else :
        pass
    import imp, sys
    
    imp.reload( sys ) ;
    _encoding_str = 'Default encoding changed from {_encoding_from} to {_encoding_to}'.format( _encoding_from=sys.getdefaultencoding(), _encoding_to='{_encoding_to}' )
    sys.setdefaultencoding( "utf8" )
    print( _encoding_str.format( _encoding_to=sys.getdefaultencoding() ) )



# -*- coding: utf-8 -*-
# @Time    : 2019/11/19 20:19
# @Author  : Dong Qirui
# @Software: PyCharm

from datetime import datetime, timedelta
from enum import Enum


class DateTimeFormat( Enum ) :
    '''
    
    '''
    DATETIME_FULL = '%Y-%m-%d %H:%M:%S'
    DATETIME_SHORT = '%Y-%-m-%-d %-H:%-M:%-S'
    DATE_FULL = '%Y-%m-%d'
    DATE_SHORT = '%Y-%-m-%-d'
    DATE_DIGITAL = '%Y%m%d'
    MONTH_FULL = '%Y-%m'


def getDateTime ( delta: int = 0 ) -> datetime :
    '''
    获取日期，明天：1；昨天：-1
    :return: datetime.datetime(2020, 11, 2, 21, 28, 22, 772748)
    '''
    return datetime.today() + timedelta( days=+delta )


def parseDateTime ( datetime_str: str, format_str: str ) -> datetime :
    '''
    从时间字符串转换时间
    :return: datetime.datetime(2020, 11, 2, 21, 28, 22, 772748)
    '''
    return datetime.strptime( datetime_str, format_str )


def getDateStrFormatDigital ( delta: int = 0 ) -> str :
    '''
    
    :return: '20190508'
    '''
    
    return getDateTimeStrFormatFull( delta, DateTimeFormat.DATE_DIGITAL )


def getDateStrFormatFull ( delta: int = 0 ) -> str :
    '''
    
    :return: '2019-05-08'
    '''
    return getDateTimeStrFormatFull( delta, DateTimeFormat.DATE_FULL )


def getDateStrFormatShort ( delta: int = 0 ) -> str :
    '''
    
    :return: '2019-5-8'
    '''
    return getDateTimeStrFormatFull( delta, DateTimeFormat.DATE_SHORT )


def getDateTimeStrFormat ( delta: int = 0, format_str: str = DateTimeFormat.DATETIME_FULL.value ) -> str :
    '''
    based on format_enum
    :return: '2019-05-08 16:43:18'
    '''
    return getDateTime( delta ).strftime( format_str )


def getDateTimeStrFormatEnum ( delta: int = 0, format_enum: DateTimeFormat = DateTimeFormat.DATETIME_FULL ) -> str :
    '''
    based on format_enum
    :return: '2019-05-08 16:43:18'
    '''
    return getDateTime( delta ).strftime( format_enum.value )


def getDateTimeStrFormatFull ( delta: int = 0 ) -> str :
    '''
    based on format_enum
    :return: '2019-05-08 16:43:18'
    '''
    return getDateTimeStrFormatFull( delta, DateTimeFormat.DATETIME_FULL )


def getDateTimeStrFormatShort ( delta: int = 0 ) -> str :
    '''
    based on format_enum
    :return: '2019-5-8 16:43:18'
    '''
    return getDateTimeStrFormatFull( delta, DateTimeFormat.DATETIME_SHORT )


def getMonthStrFormat ( delta: int = 0 ) -> str :
    '''
    
    :return: '2019-05'
    '''
    return getDateTimeStrFormatFull( delta, DateTimeFormat.MONTH_FULL )
